package bankgui;

import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.event.KeyEvent;

import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JButton;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;

import java.util.List;
import java.util.Vector;

import bank.Account;
import bank.CentralHost;
import bank.PermanentAccount;

public class AccountsPanel extends JPanel {


	/**
	 * Class for making account data of a <code>CentralHost</code> accessible
	 * via Swing's <code>TableModel</code> interface. It also caches the data
	 * and thus needs to be synchronised when the data changes.
	 *
	 */
	private class AccountsTableModel extends AbstractTableModel {

		private final String columnNames[] = {"Account No.", "Balance"};
		private CentralHost host;
		private List accountList = new Vector();

		/**
		 * @param host the central host storing the accounts for this model
		 */
		public AccountsTableModel (CentralHost host) {
			this.host = host;
			rereadData();
		}

		public int getColumnCount() {
			return 2;
		}

		public int getRowCount() {
			return accountList.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			Account account = (Account) accountList.get(rowIndex);

			switch (columnIndex) {
				case 0: return new Integer(account.getAccountNumber());
				case 1: return new Integer(account.accountBalance());
			}
			return null;
		}

		public String getColumnName(int column) {
			if (column < 0 || column >= 2)
				return null;
			return columnNames[column];
		}

		/**
		 * Re-read the accounts from the central host.
		 */
		public void rereadData() {
			accountList.clear();
			for (int i = 0; i < host.maxAccountNumber; i++)
				if (host.getAccount(i) != null)
					accountList.add(host.getAccount(i));
		}

		/**
		 * @param index row index for the table 
		 * @return the account with index <code>index</code>
		 * @throws IndexOutOfBoundsException if <code>index</code> is out of
		 * 			bounds (<code>index < 0 || index >= getRowCount()</code>)
		 */
		public PermanentAccount getAccountByIndex (int index) throws IndexOutOfBoundsException {
			return (PermanentAccount) accountList.get(index);
		}
	}


	private AccountsTableModel accountsData;
	private CentralHost host;
	private BankModelInteraction modelInteraction;

	private static final long serialVersionUID = 1L;
	private JScrollPane accountsScrollPane = null;
	private JTable accountsTable = null;
	private JButton depositButton = null;
	private JButton infoButton = null;
	private JPanel buttonPanel = null;

	/**
	 * @param host the central host storing the accounts to be displayed
	 */
	public AccountsPanel (CentralHost host, BankModelInteraction modelInteraction) {
		super();
		this.host = host;
		this.modelInteraction = modelInteraction;
		this.accountsData = new AccountsTableModel(this.host);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
		gridBagConstraints11.gridx = 0;
		gridBagConstraints11.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints11.gridy = 1;
		GridBagConstraints accountsScrollPaneConstraints = new GridBagConstraints();
		accountsScrollPaneConstraints.fill = GridBagConstraints.BOTH;
		accountsScrollPaneConstraints.weighty = 1.0;
		accountsScrollPaneConstraints.gridx = 0;
		accountsScrollPaneConstraints.gridy = 0;
		accountsScrollPaneConstraints.weightx = 1.0;
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(getAccountsScrollPane(), accountsScrollPaneConstraints);
		this.add(getButtonPanel(), gridBagConstraints11);
	}

	/**
	 * This method initializes accountsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getAccountsScrollPane() {
		if (accountsScrollPane == null) {
			accountsScrollPane = new JScrollPane();
			accountsScrollPane.setViewportView(getAccountsTable());
		}
		return accountsScrollPane;
	}

	/**
	 * This method initializes accountsTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getAccountsTable() {
		if (accountsTable == null) {
			accountsTable = new JTable(accountsData);
			accountsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			accountsTable.getSelectionModel().addListSelectionListener(new javax.swing.event.ListSelectionListener() {
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
                    if (accountsTable.getSelectedRow() == -1) {
                        depositButton.setEnabled(false);
                        infoButton.setEnabled(false);
                    }
                    else {
                    	depositButton.setEnabled(true);
                        infoButton.setEnabled(true);
                    }
				}
			});
		}
		return accountsTable;
	}

	/**
	 * This method initializes depositButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDepositButton() {
		if (depositButton == null) {
			depositButton = new JButton();
			depositButton.setEnabled(false);
			depositButton.setMnemonic(KeyEvent.VK_D);
			depositButton.setText("Deposit");
			depositButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					int index = accountsTable.getSelectedRow();
					if (index == -1)
						return;
					PermanentAccount account = accountsData.getAccountByIndex(index);

					String errorMessage = null;
					String amountString = JOptionPane.showInputDialog(getMainComponent(),
							"Amount to deposit to account no. " + account.getAccountNumber() + "):",
                            "Deposit", JOptionPane.PLAIN_MESSAGE);
					if (amountString == null)
						return;
					try {
						int amount = Integer.parseInt(amountString);
						if (amount <= 0)
							errorMessage = "Amount must be positive.";
						else if (account.accountBalance() + amount <= account.accountBalance())
							errorMessage = "Amount would cause an overflow.";
						else
							account.deposit(amount);
					} catch (NumberFormatException exc) {
						errorMessage = "Invalid input (" + exc.getMessage() + ").";
					}
					if (errorMessage != null)
						JOptionPane.showMessageDialog(getMainComponent(),
								errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
					else
						modelInteraction.accountsChanged();
				}
			});
		}
		return depositButton;
	}

	/**
	 * This method initializes infoButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getInfoButton() {
		if (infoButton == null) {
			infoButton = new JButton();
			infoButton.setText("Info ...");
			infoButton.setMnemonic(KeyEvent.VK_N);
			infoButton.setEnabled(false);
			infoButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					int index = accountsTable.getSelectedRow();
					if (index == -1)
						return;
					Account account = accountsData.getAccountByIndex(index);

					JOptionPane.showMessageDialog(getMainComponent(),
                    		new AccountInfoPanel(account, modelInteraction),
                            "Account information", JOptionPane.PLAIN_MESSAGE);
				}
			});
		}
		return infoButton;
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.setLayout(new FlowLayout());
			buttonPanel.add(getDepositButton(), null);
			buttonPanel.add(getInfoButton(), null);
		}
		return buttonPanel;
	}

	/**
	 * Update the view of the accounts (if the set of accounts in the central
	 * host has changed). 
	 */
	void updateAccounts() {
		accountsData.rereadData();
		accountsTable.updateUI();
	}

	private Component getMainComponent() {
		return this;
	}
}
